.ckanext-jstree-search{margin-bottom:12px;display:flex;gap:12px}.ckanext-jstree-search .jstree-search--input{display:flex;position:relative;flex-basis:350px;height:35px}.jstree-default .jstree-search{font-style:normal !important}.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl,.jstree-default>.jstree-no-dots .jstree-closed .jstree-ocl{background-image:unset;font-style:inherit}.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl:before,.jstree-default>.jstree-no-dots .jstree-closed .jstree-ocl:before{font-family:var(--fa-style-family, "Font Awesome 6 Free");font-weight:var(--fa-style, 900)}.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl{background-image:unset}.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl:before{content:""}.jstree-default>.jstree-no-dots .jstree-closed .jstree-ocl:before{content:""}.jstree-default>.jstree-no-dots .jstree-leaf>.jstree-ocl:before{display:none}.jstree-default .jstree-search{background:bisque}.jstree-default .jstree-anchor[aria-expanded=true] .jstree-icon:before{content:""}#archive-tree{margin-bottom:1rem;height:700px;overflow-y:scroll}#archive-tree-error{padding:10px;border:1px solid #ced4da;border-radius:.25rem}.jstree-anchor{display:inline-flex !important;width:97% !important}.jstree-anchor .unfold-node-metadata{margin-left:auto}.jstree-anchor .unfold-node-metadata .unfold-node-size{margin-right:1rem}.pseudo-table-header{display:flex;font-weight:bold;padding:.25rem .75rem;border-bottom:1px solid #ced4da;margin-bottom:.5rem}.pseudo-table-header .unf-cell-size{margin-left:auto;min-width:90px}.pseudo-table-header .unf-cell-modified-at{min-width:160px}ul.vakata-context{z-index:5;padding:0;min-width:150px;font-size:1rem;background:var(--bs-white);box-shadow:var(--bs-box-shadow);border:0;border-radius:3px}ul.vakata-context li{padding:0;border:0}ul.vakata-context li a{padding:0 1.2rem;border:0;background:rgba(0,0,0,0)}ul.vakata-context li a:hover{background-color:var(--bs-light)}ul.vakata-context li a i,ul.vakata-context li a .vakata-contextmenu-sep{display:none}ul.vakata-context li a ins,ul.vakata-context li a span{display:none;border:0 !important}ul.vakata-context li ul{background:var(--bs-white);box-shadow:var(--bs-box-shadow);border:0;border-radius:3px}ul.vakata-context li ul li{padding:0;border:0}ul.vakata-context li ul li a{padding:0 1.2rem;border:0;background:rgba(0,0,0,0)}ul.vakata-context li ul li a:hover{background-color:var(--bs-light)}ul.vakata-context li ul li a i,ul.vakata-context li ul li a .vakata-contextmenu-sep{display:none}ul.vakata-context li ul li a ins,ul.vakata-context li ul li a span{display:none;border:0 !important}ul.vakata-context .vakata-context-separator a{margin:0;border:0;height:2px;background-color:#d0d5dd}